const path = require('path');
const chalk = require('chalk');
const _ = require('lodash');
const { readdirSync, statSync, existsSync, closeSync, openSync } = require('fs');
const paths = require('./paths');
const joinPath = path.join;

const isDirectory = file => statSync(file).isDirectory();
const isFile = file => !isDirectory(file);

const findClientFilesPaths = (fragmentDir, ext = 'js') => {
  try {
    return readdirSync(fragmentDir)
      .filter(dirname => isDirectory(joinPath(fragmentDir, dirname)))
      .map(dirname => ({
        name: dirname,
        pathname: joinPath(fragmentDir, dirname, 'client', `index.${ext}`),
      }));
  } catch (e) {
    return [];
  }
};

module.exports.fragmentClientEntries = fragmentDir => {
  const files = findClientFilesPaths(fragmentDir, 'js')
    .concat(findClientFilesPaths(fragmentDir, 'jsx'));
  if (!files) {
    return {};
  }
  return files
    .filter(({ pathname }) => existsSync(pathname))
    .reduce((entries, { name, pathname }) => {
      entries[name] = pathname;
      return entries;
    }, {});
};

module.exports.fragmentStaticPagesClientEntries = fragmentDir => {
  const files = findClientFilesPaths(fragmentDir, 'js')
    .concat(findClientFilesPaths(fragmentDir, 'jsx'));
  if (!files) {
    return {};
  }
  return files
    .reduce((entries, { name, pathname }) => {
      if (existsSync(pathname)) {
        entries[name] = pathname;
      } else {
        const subDir = joinPath(fragmentDir, name);
        const subFiles = findClientFilesPaths(subDir);
        if (subFiles) {
          subFiles.forEach(({ name: childName, pathname }) => {
            if (existsSync(pathname)) {
              entries[`${name}-${childName}`] = pathname;
            }
          });
        }
      }
      return entries;
    }, {});
};

module.exports.lambdaServerHandlers = lambdasDir => {
  if (!existsSync(lambdasDir)) {
    return {};
  }
  const lambdaFiles = readdirSync(lambdasDir)
    .map(file => joinPath(lambdasDir, file))
    .filter(isFile);

  return lambdaFiles.reduce((entries, lambda) => {
    entries[path.basename(lambda, '.js')] = lambda;
    return entries;
  }, {});
};

module.exports.handleCompilationError = (err, stats) => {
  const statsData = stats.toJson();

  if (err || stats.hasErrors()) {
    console.error(chalk.red('Build failed'), err || statsData.errors.join());
    process.exit(1);
  }
};

module.exports.checkAssetManifest = () => {
  const assetManifestFile = joinPath(paths.root, 'asset-manifest.json');

  if (!existsSync(assetManifestFile)) {
    closeSync(openSync(assetManifestFile, 'w'));
  }
};
