/* eslint-disable import/no-extraneous-dependencies */
const path = require('path');
const WebpackAssetsManifest = require('webpack-assets-manifest');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const Dotenv = require('dotenv-webpack');
const paths = require('./paths');
const { fragmentClientEntries, fragmentStaticPagesClientEntries } = require('./utils');
const { assetsBaseUrl, externals, symlinksLibrary } = require('./conf');

/**
 * Loader rule for js source files client
 */
const jsRule = {
  test: /\.(js|jsx)$/,
  exclude: /(node_modules)/,
  use: {
    loader: require.resolve('babel-loader'),
    options: {
      presets: [
        // Use builtins polyfill babel-polyfill
        // https://babeljs.io/docs/en/babel-polyfill
        [
          require.resolve('@babel/preset-env'),
          {
            useBuiltIns: 'usage',
            corejs: '2',
            targets: {
              ie: '11'
            }
          }
        ],
        require.resolve('@babel/preset-react')
      ],
      plugins: [
        require.resolve('@babel/plugin-proposal-object-rest-spread'),
        require.resolve('@babel/plugin-syntax-async-generators'),
        require.resolve('@babel/plugin-proposal-class-properties')
      ]
    }
  }
};

const jsRuleClient = {
  ...jsRule
};

const jsRuleServer = {
  ...jsRule,
  exclude: [/(node_modules)/, /.*src(\/|\\)fragments(\/|\\).*(\/|\\)assets(\/|\\).*/]
};

/**
 * Loader rule for source map
 */
const sourceMapRule = {
  test: /\.js$/,
  use: [require.resolve('source-map-loader')],
  enforce: 'pre'
};

/**
 * Loader rule for css
 */
const cssRule = {
  test: /\.(css|scss)$/,
  use: [
    /**
     * TODO: this break user-classified-data
     */
    MiniCssExtractPlugin.loader,
    {
      loader: require.resolve('css-loader'),
      options: {
        importLoaders: 1
      }
    },
    {
      loader: require.resolve('postcss-loader'),
      options: {
        plugins: () => [
          require('postcss-preset-env')({
            autoprefixer: {
              flexbox: 'no-2009'
            },
            stage: 3
          })
        ]
      }
    },
    require.resolve('sass-loader')
  ]
};

/**
 * Loader rule for fonts
 */
const fontRule = {
  test: /\.(woff(2)?)$/i,
  use: [
    {
      loader: require.resolve('file-loader'),
      options: {
        name: '[name]-[hash:8].[ext]',
        publicPath: filename => `${assetsBaseUrl}/${filename}`
      }
    }
  ]
};

/**
 * Loader rule for image
 */
const imageRule = {
  test: /\.(gif|png|jpe?g|svg|pdf)$/i,
  use: [
    {
      loader: require.resolve('file-loader'),
      options: {
        name: '[name]-[hash:8].[ext]',
        publicPath: filename => `${assetsBaseUrl}/${filename}`
      }
    }
  ]
};

const imageRuleClient = {
  ...imageRule
};

const imageRuleServer = {
  ...imageRule,
  exclude: [/.*src(\/|\\)fragments(\/|\\).*(\/|\\)assets(\/|\\).*/],
  use: [
    {
      loader: require.resolve('file-loader'),
      options: {
        name: '[name]-[hash:8].[ext]',
        publicPath: filename => `${assetsBaseUrl}/${filename}`,
        outputPath: 'static'
      }
    }
  ]
};

/**
 * Loader rule to import file as string
 */
const rawRule = {
  test: /(\\|\/)env\./,
  use: require.resolve('raw-loader')
};

/**
 * Loader rule for handlebars file
 */
const handlebarsRule = {
  test: /\.hbs/,
  loader: require.resolve('handlebars-loader'),
  options: {
    precompileOptions: {
      knownHelpersOnly: false
    }
  }
};

/**
 * Loader rule for assets
 */
const assetsRule = {
  test: [/.*src(\/|\\)fragments(\/|\\).*(\/|\\)assets(\/|\\).*/],
  use: [
    {
      loader: require.resolve('file-loader'),
      options: {
        name: '[name]-[hash:8].[ext]',
        publicPath: filename => `${assetsBaseUrl}/${filename}`,
        outputPath: 'static'
      }
    }
  ]
};

const nullLoader = {
  test: /\.(css|scss)$/,
  use: [require.resolve('null-loader')]
};

/**
 * Client bundle configuration
 */
module.exports.client = {
  devtool: 'source-map',
  entry: {
    ...fragmentClientEntries(path.resolve(paths.src, 'fragments')),
    ...fragmentStaticPagesClientEntries(path.resolve(paths.src, 'fragments', 'static-page', 'pages')),
  },
  output: {
    path: paths.assets,
    filename: '[name]-[hash:8].js',
    libraryTarget: 'amd',
    devtoolModuleFilenameTemplate: info =>
      path
        .relative(paths.src, info.absoluteResourcePath)
        .replace(/\\/g, '/'),
  },
  module: {
    rules: [sourceMapRule, jsRuleClient, cssRule, fontRule, imageRuleClient]
  },
  externals,
  plugins: [
    new WebpackAssetsManifest({
      output: path.resolve(paths.root, 'asset-manifest.json'),
      publicPath: `${assetsBaseUrl}/`,
      merge: true
    }),
    new MiniCssExtractPlugin({
      filename: '[name]-[hash:8].css'
    }),
    new Dotenv({
      path: path.join(paths.config, `./browser/env.${process.env.ENV || 'local'}`)
    })
  ],
  resolve: {
    symlinks: symlinksLibrary,
    extensions: ['.js', '.jsx']
  }
};

/**
 * Server bundle configuration
 */
module.exports.server = {
  output: {
    libraryTarget: 'commonjs2',
    path: paths.build,
    filename: '[name].js'
  },
  target: 'node',
  node: {
    __dirname: false,
    __filename: true
  },
  optimization: {
    minimize: false
  },
  module: {
    rules: [sourceMapRule, jsRuleServer, nullLoader, rawRule, handlebarsRule, imageRuleServer, assetsRule]
  },
  externals: {
    'aws-sdk': 'aws-sdk'
  },
  plugins: [
    new Dotenv({
      path: path.join(paths.config, `./server/env.${process.env.ENV || 'local'}`)
    }),
    new MiniCssExtractPlugin({
      filename: '[name]-[hash:8].css'
    })
  ],
  resolve: {
    symlinks: symlinksLibrary,
    extensions: ['.js', '.jsx']
  }
};

/**
 * Commonns libs bundle configuration
 */
module.exports.commons = {
  devtool: 'source-map',
  output: {
    path: paths.assets,
    filename: '[name]-[hash:8].js',
    libraryTarget: 'umd'
  },
  optimization: {},
  module: {
    rules: [sourceMapRule, jsRule, cssRule]
  },
  plugins: [
    new WebpackAssetsManifest({
      output: path.resolve(paths.root, 'asset-manifest.json'),
      publicPath: `${assetsBaseUrl}/`,
      merge: true
    }),
    new MiniCssExtractPlugin({
      filename: '[name]-[hash:8].css'
    }),
    new Dotenv({
      path: path.join(paths.config, `./browser/env.${process.env.ENV || 'local'}`)
    })
  ],
  resolve: {
    symlinks: symlinksLibrary
  }
};
