const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const TerserJSPlugin = require('terser-webpack-plugin');

const WebpackAssetsManifest = require('webpack-assets-manifest');
const Dotenv = require('dotenv-webpack');

/**
 * Babelize the source code
 * @param {*} options
 */
const babelize = (options = {}) => {
  const { exclude = /node_modules/ } = options;

  return {
    module: {
      rules: [
        {
          test: /\.js$/,
          exclude,
          use: {
            loader: require.resolve('babel-loader'),
            options: {
              presets: [
                // Use builtins polyfill babel-polyfill
                // https://babeljs.io/docs/en/babel-polyfill
                [
                  require.resolve('@babel/preset-env'),
                  {
                    modules: false,
                    corejs: "2",
                    useBuiltIns: 'usage',
                    targets: {
                      ie: '11',
                    },
                  },
                ],
                require.resolve('@babel/preset-react'),
              ],
              plugins: [
                require.resolve('@babel/plugin-proposal-object-rest-spread'),
                require.resolve('@babel/plugin-syntax-async-generators'),
                require.resolve('@babel/plugin-proposal-class-properties'),
              ],
            },
          },
        },
      ],
    },
  };
};

/**
 * Extract CSS
 * @param {*} options
 */
const extractCss = (options = {}) => {
  const { filename } = options;

  return {
    plugins: [
      new MiniCssExtractPlugin({
        filename,
      }),
    ],
  };
};

/**
 * Process CSS
 * @param {*} options
 */
const processCss = (options = {}) => {
  return {
    module: {
      rules: [
        {
          test: /\.(css|scss)$/,
          use: [
            MiniCssExtractPlugin.loader,
            {
              loader: require.resolve('css-loader'),
              options: {
                importLoaders: 1,
                minimize: true,
                sourceMap: true,
              },
            },
            {
              loader: require.resolve('postcss-loader'),
              options: {
                plugins: () => [
                  require('postcss-preset-env')({
                    autoprefixer: {
                      flexbox: 'no-2009',
                    },
                    stage: 3,
                  }),
                ],
                sourceMap: true,
              },
            },
            {
              loader: require.resolve('sass-loader'),
              options: {
                sourceMap: true,
              },
            },
          ],
        },
      ],
    },
  };
};

/**
 * Add dotenv file into source
 * @param {*} path
 */
const dotEnv = (options = {}) => {
  const { path } = options;

  return {
    plugins: [
      new Dotenv({
        path,
      }),
    ],
  };
};

/** */
const assetsManifest = (options = {}) => {
  const { output, publicPath } = options;

  return {
    plugins: [
      new WebpackAssetsManifest({
        output,
        publicPath,
        merge: true,
      }),
    ],
  };
};

const sourceMaps = (options = {}) => {
  const { isDev = false } = options;

  return {
    devtool: isDev ? 'cheap-module-eval-source-map ' : 'source-map',
  };
};

const assetsLoader = (options = {}) => {
  const { assetsBaseUrl } = options;

  return {
    module: {
      rules: [
        {
          test: [/.*src(\/|\\)fragments(\/|\\).*(\/|\\)assets(\/|\\).*/],
          use: [
            {
              loader: require.resolve('file-loader'),
              options: {
                name: '[name]-[hash].[ext]',
                publicPath: filename => `${assetsBaseUrl}/${filename}`,
                outputPath: 'static',
              },
            },
          ],
        },
      ],
    },
  };
};

const imagesLoader = (options = {}) => {
  const { outputPath, assetsBaseUrl, exclude } = options;

  return {
    module: {
      rules: [
        {
          test: /\.(gif|png|jpe?g|svg|pdf)$/i,
          exclude,
          use: [
            {
              loader: require.resolve('file-loader'),
              options: {
                name: '[name]-[hash].[ext]',
                publicPath: filename => `${assetsBaseUrl}/${filename}`,
                outputPath,
              },
            },
          ],
        },
      ],
    },
  };
};

const fontsLoader = (options = {}) => {
  const { assetsBaseUrl } = options;

  return {
    module: {
      rules: [
        {
          test: /\.(woff(2)?)$/i,
          use: [
            {
              loader: require.resolve('file-loader'),
              options: {
                name: '[name]-[hash].[ext]',
                publicPath: filename => `${assetsBaseUrl}/${filename}`,
              },
            },
          ],
        },
      ],
    },
  };
};

/**
 * Handlebars loader
 * @param {*} options
 */
const handlebarsLoader = (options = {}) => {
  return {
    module: {
      rules: [
        {
          test: /\.hbs/,
          loader: require.resolve('handlebars-loader'),
          options: {
            precompileOptions: {
              knownHelpersOnly: false,
            },
          },
        },
      ],
    },
  };
};

/**
 * Load file as string
 * @param {*} options
 */
const rawLoader = (options = {}) => {
  return {
    module: {
      rules: [
        {
          test: /(\\|\/)env\./,
          use: require.resolve('raw-loader'),
        },
      ],
    },
  };
};

/**
 * No load
 * @param {*} options
 */
const nullLoader = (options = {}) => {
  const { test } = options;

  return {
    module: {
      rules: [
        {
          test,
          use: [require.resolve('null-loader')],
        },
      ],
    },
  };
};

const optimizer = (options = {}) => {
  return {
    optimization: {
      minimizer: [
        new TerserJSPlugin({ sourceMap: true }),
        new OptimizeCSSAssetsPlugin({
          cssProcessorOptions: {
            map: {
              inline: false,
            },
          },
        }),
      ],
    },
  };
};

module.exports = {
  babelize,
  dotEnv,
  extractCss,
  processCss,
  assetsManifest,
  sourceMaps,
  assetsLoader,
  imagesLoader,
  fontsLoader,
  handlebarsLoader,
  rawLoader,
  nullLoader,
  optimizer,
};
